#ifdef GL_ES
#define LOWP lowp
precision mediump float;
#else
#define LOWP
#endif

varying LOWP vec4 v_color;
varying vec2 v_texCoords;
varying float v_type;

uniform sampler2D u_texture;
uniform sampler2D u_texture1;
uniform vec2 uv_regionoffset;
uniform vec2 uv_regionsize;
uniform float time;

vec4 back_color = vec4(0,0,0,1);
float glow_border = 0.08;
float glow_radius = 0.1;
float glow2_border = 0.2;
float glow2_radius = 0.3;
float core_border = 0.03;
float core_radius = 0.04;
vec2 disc_center = vec2(0.5, 0.5);
float disc_radius = 0.45;
float border_size = 0.002;
float rspeed = 3.0;

// texture atlas stuff
float coreRegularDownscale = 6.3;
vec2 coreRegularOffset = vec2(0.0,0);
float coreBigDownscale = 2.1;
vec2 coreBigOffset = vec2(0.5,0);
vec2 sprites = vec2(2.0,1.0);


float oscillate(float vMin, float vMax, float t) {
    float halfRange = (vMax - vMin) / vMax * 0.5;
    return (sin(t) * halfRange + (1.0 - halfRange)) * vMax;
}

float makeDisc(vec2 p, float b, float r) {
	return smoothstep(r + b, r - b, sqrt(dot(p, p)));
}

float makeCircle(vec2 p, float b, float r) {
    float dist =  sqrt(dot(p, p));
    return  1.0 + smoothstep(r, r+b, dist) - smoothstep(r-b, r, dist);
}

vec2 rot(float r, float deg) {
	return vec2(r * cos(radians(deg)), r * sin(radians(deg)));
}

vec4 drawBigFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;

    vec2 corepos = v_texCoords;
    corepos -= disc_center;
    corepos.x += sin(4.0 * rndt) * 0.10;
    corepos.y += cos(4.0 * rndt) * 0.10;
    vec4 coredisk = mix(back_color, vec4(1,1,1,1), makeDisc(corepos, 0.1, oscillate(0.12,0.15, rndt*5.5))) * oscillate(0.1, 0.6, (t+v_color.a) * 12.5);

    vec2 glow1pos = v_texCoords;
    glow1pos -= disc_center;
    vec4 glow1disk = mix(back_color, vec4(v_color.rgb * 0.27, 1), makeDisc(glow1pos, 0.2, 0.3)) * oscillate(0.0, 1.0, (t+v_color.a) * 12.5);

    vec2 glow2pos = v_texCoords;
    glow2pos -= disc_center;
    glow2pos.x += sin(4.0 * rndt) * 0.11;
    glow2pos.y += cos(4.0 * rndt) * 0.11;
    vec4 glow2disk = mix(glow1disk, vec4(v_color.rgb * 0.62 , 1), makeDisc(glow2pos, 0.04, oscillate(0.13,0.18, (t+v_color.a) * 12.5))) ;

    vec2 texuv = v_texCoords * vec2(1.0, 1.0)/sprites + coreBigOffset;
    texuv = texuv * coreBigDownscale - vec2(coreBigDownscale/2.0 - 0.5  ,coreBigDownscale/2.0 - 0.5) - (coreBigOffset*coreBigDownscale - coreBigOffset)/2.0 ;
    texuv.x += sin(4.0 * rndt) * 0.13;
    texuv.y += cos(4.0 * rndt) * 0.13;
    vec4 tex = texture2D(u_texture, clamp(texuv, vec2(0.5,0.0), vec2(1.0,1.0))) * vec4(v_color.rgb,1.0);

     return glow2disk + tex + coredisk;
}

vec4 drawSmallFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;

    vec2 glow1pos = v_texCoords;
    glow1pos -= disc_center;
    vec4 glow1disk = mix(back_color, vec4(v_color.rgb * 0.18, 1), makeDisc(glow1pos, 0.25, 0.35)) * oscillate(0.6, 1.0, (t+v_color.a) * 11.5);

    vec2 glow2pos = v_texCoords;
    glow2pos -= disc_center;
    glow2pos.x -= sin(2.0 * rndt) * 0.12;
    glow2pos.y -= cos(2.0 * rndt) * 0.12;
    vec4 glow2disk = mix(back_color, vec4(v_color.rgb * 0.8 , 1), makeDisc(glow2pos, 0.2, 0.1)) * oscillate(0.3, 1.0, (t+v_color.a) * 11.5);

    vec2 corepos = v_texCoords;
    corepos -= disc_center;
    corepos.x -= sin(2.0 * rndt) * 0.13;
    corepos.y -= cos(2.0 * rndt) * 0.13;
    vec4 coredisk = mix(back_color, vec4(1,1,1,1), makeDisc(corepos, 0.03, 0.1))* 0.5;

    return coredisk + glow2disk + glow1disk ;
}

vec4 drawToxicFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;
    float tt = t * v_color.a;
    float toxicTexDownscale = 1.0;
    float rotspeed = 0.3;

    float x = (disc_center.x - v_texCoords.x);
	float y = (disc_center.y - v_texCoords.y);
    float r = -(x*x + y*y) * 1.5;
    float z = 1.0 + 0.5*sin((r + tt*0.1)/0.026);
    z = pow(z, 3.0);

    vec2 tex0uv = v_texCoords;
    tex0uv = tex0uv * vec2(1.0, 1.0)/sprites ;
    tex0uv = tex0uv * toxicTexDownscale - vec2(toxicTexDownscale/2.0 - 0.5  ,toxicTexDownscale/2.0 - 0.5) + (vec2(0.5,0.0)*toxicTexDownscale -  vec2(0.5,0.0))/2.0  ;
    tex0uv.x += sin(2.1 * rndt) * 0.03;
    tex0uv.y += cos(2.1 * rndt) * 0.03;

    vec2 toxicglowpos = v_texCoords-disc_center;
    toxicglowpos.x += sin(2.1 * rndt) * 0.03;
    toxicglowpos.y += cos(2.1 * rndt) * 0.03;
    vec4 toxicglow = mix(back_color, vec4(z, z, z, 1.0), makeDisc(toxicglowpos, 0.2, 0.2))* 0.5;

    vec2 corepos = v_texCoords;
    corepos -= disc_center;
    corepos.x += sin(3.0 * rndt) * 0.03;
    corepos.y += cos(3.0 * rndt) * 0.03;
    vec4 coredisk = mix(back_color, vec4(1.0,1.0,1.0,1.0), makeDisc(corepos, 0.03, 0.1))* 0.6;

    vec4 tex0 = texture2D(u_texture,  clamp(tex0uv, vec2(0.0, 0.0), vec2(0.5, 1.0)));

    return tex0 * toxicglow * vec4(0.0, 1.0, 0.13, 1.0) + coredisk ;
}

void main()
{
    float new_t = time * v_color.a; // time randomizer


    if (v_type == 0.0) {
        gl_FragColor = drawBigFood(time);// big food

    } else if (v_type == 1.0) {
        gl_FragColor = drawSmallFood(time);// regular food

    } else if (v_type == 2.0) {
        gl_FragColor = drawToxicFood(time);

    } else {
        discard;//gl_FragColor = vec4(v_texCoords.x, v_texCoords.y,0.0,oscillate(0.3,0.6,new_t));
    }

}